import { GraphQLError, Kind } from 'graphql';
import { createGraphQLError, isDocumentNode } from '@graphql-tools/utils';
export function assertMutationViaGet(method, document) {
    const isMutation = document?.definitions.find(def => def.kind === Kind.OPERATION_DEFINITION)?.operation ===
        'mutation';
    if (isMutation && method === 'GET') {
        throw createGraphQLError('Can only perform a mutation operation from a POST request.', {
            extensions: {
                http: {
                    status: 405,
                    headers: {
                        Allow: 'POST',
                    },
                },
            },
        });
    }
}
export function usePreventMutationViaGET() {
    return {
        onParse() {
            // We should improve this by getting Yoga stuff from the hook params directly instead of the context
            return ({ result, context: { request } }) => {
                // Run only if this is a Yoga request
                // the `request` might be missing when using graphql-ws for example
                // in which case throwing an error would abruptly close the socket
                if (!request) {
                    return;
                }
                if (result instanceof Error) {
                    if (result instanceof GraphQLError) {
                        result.extensions.http = {
                            spec: true,
                            status: 400,
                        };
                    }
                    throw result;
                }
                if (isDocumentNode(result)) {
                    assertMutationViaGet(request.method, result);
                }
            };
        },
    };
}
